;;; This file is only used for installing AUCTeX.

;; Copyright (C) 2013 Free Software Foundation, Inc.

;;; It is not a part of AUCTeX itself.

;; Make sure we get the right files.
(setq load-path (cons "." load-path)
      TeX-lisp-directory "<none>"
      TeX-auto-global "<none>")
